/**
 * Copyright (c) 2008, Mats Lindh (http://e-mats.org/)
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **/
package org.emats
{
    /* Imports */
    import mx.containers.Box;
    import mx.effects.*;
    import mx.controls.Label;
    import mx.events.FlexEvent;
    import flash.net.FileReference;

    /**
     * Represents a progress bar for a file upload.
     **/
    public class FileUploadProgressBar extends Box
    {
        /**
         * The box that will be used as the "we've uploaded this much" bar.
         **/
        private var progressSoFar:Box = new Box();

        /**
         * The box that will be used as the "this is still missing" bar.
         **/
        private var progressToGo:Box = new Box();

        /**
         * The label that holds the filaname of the file being uploaded.
         **/
        private var filenameLabel:Label = new Label();

        /**
         * The label that holds how many percent of the upload that's done.
         **/
        private var percentDone:Label = new Label();

        /**
         * The width of the progressbar in pixels.
         **/
        private var progressIndicatorWidth:int = 300;

        /**
         * The background to use for the progress indicator box.
         **/
        private var progressIndicatorColor:String = "#bfbfbf";

        /**
         * The background to use for the "still left" box.
         **/
        private var progressIndicatorBackgroundColor:String = "white";

        /**
         * The filename of the file this progress bar represents.
         **/
        private var filename:String;

        /**
         * Constructor for the class.
         *
         * @param filename The filename of the file this progressbar tracks.
         **/
        public function FileUploadProgressBar(filename:String)
        {
            this.filename = filename;

            // create effects for hiding the progressbar
            var fadeEffect:Fade = new Fade();
            var resizeEffect:Resize = new Resize();

            // we use a parallel effect to make the progress bar disappear and fade out simultanously
            var hideEffect:Parallel = new Parallel();

            // set animation parameters
            resizeEffect.duration = 800;
            resizeEffect.heightTo = 0;
            fadeEffect.duration = 800;

            // add effects in parallel
            hideEffect.addChild(resizeEffect);
            hideEffect.addChild(fadeEffect);

            // create the container for all the elements for a file upload
            this.direction = "horizontal";
            this.setStyle("horizontalGap", 0);
            this.setStyle("hideEffect", hideEffect);

            // set the style and settings for the label for the filename
            filenameLabel.text = this.filename;
            filenameLabel.width = 200;
            filenameLabel.maxWidth = 200;
            this.addChild(filenameLabel);

            // set the properties for the box that shows how much we've uploaded.
            progressSoFar.direction = "horizontal";

            // set style for progress box
            progressSoFar.setStyle("backgroundColor", this.progressIndicatorColor);
            progressSoFar.setStyle("borderStyle", "solid");
            progressSoFar.setStyle("borderThickness", 2);
            progressSoFar.setStyle("borderSides", "right");
            progressSoFar.height = 20;
            progressSoFar.width = 1;

            this.addChild(progressSoFar);

            // set properties for the box that shows how much we have left to upload
            progressToGo.direction = "horizontal";

            // set styles for "still to go" box
            progressToGo.setStyle("backgroundColor", this.progressIndicatorBackgroundColor);
            progressToGo.setStyle("borderStyle", "solid");
            progressToGo.setStyle("borderSides", "top bottom right");
            progressToGo.height = 20;

            this.addChild(progressToGo);

            percentDone.width = 50;
            percentDone.maxWidth = 50;
            percentDone.text = "0.0%";
            percentDone.setStyle("textAlign", "right");
            this.addChild(percentDone);
        }

        /**
         * Set the width of the progress bar.
         *
         * @param width The width of the progress bar in pixels.
         **/
        public function setProgressIndicatorWidth(width:int):void
        {
            this.progressIndicatorWidth = width;
            progressToGo.width = width;
        }

        /**
         * The background of the "this much is done" bar.
         *
         * @param color The color of the bar, given in regular color codes (i.e. #ababab or white)
         **/
        public function setProgressIndicatorColor(color:String):void
        {
            this.progressIndicatorColor = color;
            progressSoFar.setStyle("backgroundColor", this.progressIndicatorColor);
        }

        /**
         * The background of the total upload progress bar.
         *
         * @param color The color of the bar, given in regular color codes (i.e. #ababab or white)
         **/
        public function setProgressIndicatorBackgroundColor(color:String):void
        {
            this.progressIndicatorBackgroundColor = color;
            progressToGo.setStyle("backgroundColor", this.progressIndicatorBackgroundColor);
        }

        /**
         * Call this function to update the amount done of the progress.
         *
         * @param factor How much of the complete upload that's finished (from 0 (not started) to 1 (complete))
         **/
        public function setProgress(factor:Number):void
        {
            progressSoFar.width = factor * this.progressIndicatorWidth;
            progressToGo.width = (1 - factor) * this.progressIndicatorWidth;
            percentDone.text = formatFactorToPercent(factor);
        }

        /**
         * Small helper function to convert the factor from setProgress to a percent string.
         *
         * @param n The amount of donage!
         **/
        private function formatFactorToPercent(n:Number):String
        {
            // * 1000 to get 0 to 1000 as value
            var stringValue:String = String(Math.round(n * 1000))

            // prepend a zero if we've not reached 1% yet..
            if (stringValue.length == 1)
            {
                stringValue = "0" + stringValue;
            }

            // insert "." as the next to last position..
            return stringValue.substr(0, stringValue.length - 1) + "." + stringValue.substr(-1) + "%";
        }
    }
}